# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright (c) 2010 Citrix Systems, Inc.
# Copyright (c) 2011 Piston Cloud Computing, Inc
# Copyright (c) 2012 University Of Minho
# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Manages information about the host OS and hypervisor.

This class encapsulates a connection to the libvirt
daemon and provides certain higher level APIs around
the raw libvirt API. These APIs are then used by all
the other libvirt related classes
"""

import os
import socket
import sys

import requests
import json
from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils


from cinder.i18n import _

LOG = logging.getLogger(__name__)
class PcenterClient(object):

    def __init__(self, api_url, username, password, verify=None, cert=None):
        self.api_url = api_url.rstrip('/')
        self.verify = verify
        self.cert = cert
        self.token = None
        adapter = requests.adapters.HTTPAdapter(pool_connections=100,
                                                pool_maxsize=1000)
        self.session = requests.Session()
        self.session.mount(self.api_url, adapter)
        self.token = self.getToken(username, password)

    def getToken(self, username, password):
        path = ('/user/token')
        request_url = '{api_url}{path}'.format(api_url=self.api_url, path=path)
        request_headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        }
        self.data = {'name': username,
                     'pwd': password}
        self.log_json_request('POST',request_url, self.data)
        response = self.session.request('POST',
                                        request_url,
                                        headers=request_headers,
                                        data=json.dumps(self.data),
                                        verify=self.verify,
                                        cert=self.cert)
        self.log_http_response(response)
        if response.status_code != 201 or response.json().get('code') != '0000':
            msg = (_(
                'Error from pcenter get token for user %(user)s.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'user': username,
                                          'status_code': response.status_code,
                                          'error':response.json().get('msg'),
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        return result.get('token')

    def _request(self, method, path, data=None):
        request_url = '{api_url}{path}'.format(api_url=self.api_url, path=path)
        request_headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'token': '%s' %self.token,
        }

        self.log_json_request(method,request_url, data)
        if data == None:
            response = self.session.request(method,
                                            request_url,
                                            headers=request_headers,
                                            verify=self.verify,
                                            cert=self.cert)
        else:
            response = self.session.request(method,
                                            request_url,
                                            headers=request_headers,
                                            data=json.dumps(data),
                                            verify=self.verify,
                                            cert=self.cert)

        self.log_http_response(response)
        return response

    @staticmethod
    def log_http_response(resp):
        status = (resp.status_code, resp.reason)
        dump = ['\nHTTP %s %s' % status]
        dump.append('')
        if resp._content:
            dump.extend([resp._content, ''])
        LOG.info('\n'.join(dump))

    def log_json_request(self, method, url, kwargs):
        curl = ['curl -i -X %s' % method ]
        if self.verify:
            curl.append('--cacert %s' % self.verify)
        if self.cert:
            curl.append('--cert %s' % self.cert)
        if kwargs:
            curl.append('-d \'%s\'' % jsonutils.dumps(kwargs))
        curl.append('-H Content-Type:application/json')
        if self.token:
            curl.append('-H token:%s' % self.token)
        curl.append('\'%s\'' % url)
        LOG.info(' '.join(curl))
	

    def get(self, path, **kwargs):
        return self._request('GET', path, data=kwargs)

    def head(self, path, kwargs):
        return self._request('HEAD', path, data=kwargs)

    def post(self, path, kwargs):
        return self._request('POST', path, data=kwargs)

    def put(self, path, kwargs):
        return self._request('PUT', path, data=kwargs)

    def delete(self, path):
        return self._request('DELETE', path)

    def patch(self, path, kwargs):
        return self._request('PATCH', path, data=kwargs)

